package nl.kallestruik.dtweaks.fakeplayer

import com.mojang.authlib.GameProfile
import net.minecraft.server.v1_16_R3.*
import nl.kallestruik.dtweaks.DTweaks
import org.bukkit.Location
import org.bukkit.craftbukkit.v1_16_R3.CraftWorld

class FakePlayer(
    minecraftServer: MinecraftServer?,
    worldServer: WorldServer?,
    gameProfile: GameProfile?,
    playerInteractManager: PlayerInteractManager?
): EntityPlayer(minecraftServer, worldServer, gameProfile, playerInteractManager) {
    val locale = "en_US"
    private var hasStartingPos = false
    private var startingX = 0.0
    private var startingY = 0.0
    private var startingZ = 0.0
    private var startingYaw = 0f
    private var startingPitch = 0f

    private constructor(
        server: MinecraftServer,
        worldIn: WorldServer,
        profile: GameProfile?,
        interactionManagerIn: PlayerInteractManager,
        x: Double,
        y: Double,
        z: Double,
        yaw: Float,
        pitch: Float
    ) : this(server, worldIn, profile, interactionManagerIn) {
        this.hasStartingPos = true
        this.startingX = x
        this.startingY = y
        this.startingZ = z
        this.startingYaw = yaw
        this.startingPitch = pitch
        applyStartingPosition()
    }

    companion object {
        fun atLocation(location: Location, name: String?): FakePlayer? {
            // Split the location into its parts for use later.
            val x = location.x
            val y = location.y
            val z = location.z
            val yaw = location.yaw.toDouble()
            val pitch = location.pitch.toDouble()
            try {
                // Get the minecraft server instance.
                val minecraftServer = MinecraftServer.getServer()
                // Create an empty variable for the world server.
                val worldServer = (location.world as CraftWorld).handle
                // Get the game profile from the cache (or retrieve it if it is not cached)
                var gameProfile = minecraftServer.userCache.getProfile(name) ?: return null
                if (gameProfile.properties.containsKey("textures")) {
                    gameProfile = TileEntitySkull.b(gameProfile, null, true).get()
                }

                // Create a player interact manager using the world server.
                val playerInteractManager = PlayerInteractManager(worldServer)

                // Create an instance of our fake player.
                val instance = FakePlayer(
                    minecraftServer, worldServer, gameProfile, playerInteractManager, x, y, z,
                    yaw.toFloat(),
                    pitch.toFloat()
                )
                // Create a fake connection for our fake player.
                val connection = FakeConnection(EnumProtocolDirection.SERVERBOUND)

                // Set access to connected channels so we can add our own connection.
                val serverConnection = minecraftServer.serverConnection

                (DTweaks.reflectionUtil.getValueFromField(
                    serverConnection!!,
                    "connectedChannels"
                ) as MutableList<NetworkManager?>).add(connection)

                // Connect the fake player to the server using the fake connection.
                FakePlayerList.a(minecraftServer.playerList, connection, instance)

                // Check if the fake player is not yet in the correct world.
                if (instance.world.dimensionKey != worldServer.dimensionKey) {
                    // Store the old world of the fake player.
                    val old_world = instance.world as WorldServer
                    // Remove the fake player from the old world.
                    old_world.removePlayer(instance)
                    // Make the fake player not be dead.
                    instance.dead = false
                    // Create the fake player in the new world.
                    worldServer.addEntity(instance)
                    // Spawn the fake player in the new world.
                    instance.spawnIn(worldServer)
                    // Move the fake player to the new world for the server.
                    minecraftServer.playerList.moveToWorld(instance, old_world, true, null, false)
                    // requestTeleport(x, y, z, yaw, pitch)
                    // Request the teleport from the fake player.
                    instance.playerConnection.a(x, y, z, yaw.toFloat(), pitch.toFloat())
                    // Set the fake player's world to the new world.
                    instance.playerInteractManager.world = worldServer
                    instance.teleportTo(worldServer, BlockPosition(x, y, z))
                }

                /// Set the fake players health to max.
                instance.health = 20.0f
                // Make the fake player not be dead.
                instance.dead = false
                // a == requestTeleport
                // Request the teleport from the fake player.
                instance.playerConnection.a(x, y, z, yaw.toFloat(), pitch.toFloat())
                // G == stepHeight
                // Set the fake players step height to 0.6 (The normal value).
                instance.G = 0.6f
                // Set the fake players gamemode to survival.
                playerInteractManager.gameMode = EnumGamemode.SURVIVAL
                // Tell everyone in the world about the fake player and where he is.
                minecraftServer.playerList.a(
                    PacketPlayOutEntityHeadRotation(instance, (instance.yaw * 256 / 360).toByte()),
                    instance.world.dimensionKey
                )
                minecraftServer.playerList.a(PacketPlayOutEntityTeleport(instance), instance.world.dimensionKey)
                // Move the fake player for the worlds chunk provider.
                instance.worldServer.getChunkProvider().movePlayer(instance)
                // bp == PLAYER_MODEL_PARTS
                instance.datawatcher.set<Byte>(bj, 0x7f.toByte()) // show all model layers (incl. capes)
                return instance
            } catch (e: Exception) {
                e.printStackTrace()
            }
            return null
        }
    }


    fun applyStartingPosition() {
        if (hasStartingPos) {
            this.setPositionRotation(startingX, startingY, startingZ, startingYaw, startingPitch)
            mot = Vec3D(0.0, 0.0, 0.0)
        }
    }


    override fun killEntity() {
        server.a(TickTask(server.ai()) { playerConnection.a(ChatComponentText("Killed")) })
    }

    override fun tick() {
        super.tick()
        if (H()) {
            I()
        }
        movementTick()
        if (server.ai() % 10 == 0) {
            playerConnection.syncPosition()
            this.worldServer.getChunkProvider().movePlayer(this)
        }
    }

    override fun die(cause: DamageSource?) {
        super.die(cause)
        health = 20f
        foodData = FoodMetaData(this)
        killEntity()
    }
}