package nl.kallestruik.dtweaks.tweaks.miscellaneoustweaks

import nl.kallestruik.dtweaks.managers.PocketDimensionManager
import nl.kallestruik.dtweaks.tweaks.ITweak
import org.bukkit.entity.Player
import org.bukkit.event.EventHandler
import org.bukkit.event.HandlerList
import org.bukkit.event.Listener
import org.bukkit.event.block.BlockBreakEvent
import org.bukkit.event.entity.CreatureSpawnEvent
import org.bukkit.event.entity.EntityDamageEvent
import org.bukkit.event.player.PlayerMoveEvent
import org.bukkit.plugin.java.JavaPlugin

class SpaceTimePockets(
    private val plugin: JavaPlugin,
    private val pocketDimensionManager: PocketDimensionManager
): ITweak, Listener {
    override fun getIdentifier(): String = "SpaceTimePockets"
    override fun getCategories(): List<String> = listOf("miscellaneous", "survival")

    override fun onRegister() {
        plugin.server.pluginManager.registerEvents(this, plugin)
    }

    override fun onUnRegister() {
        HandlerList.unregisterAll(this)
    }


    @EventHandler
    fun onBlockBreak(event: BlockBreakEvent) {
        if (event.player.world != pocketDimensionManager.pocketDimension) {
            return
        }
        if (event.block.location.chunk == event.player.location.chunk && event.block.y != 0 && event.block.y != 255) return
        event.isCancelled = true
        event.player.sendMessage("It seems like a bad idea to break the only thing keeping you from getting lost into the endlessness around you.")
    }

    @EventHandler
    fun onMobSpawn(event: CreatureSpawnEvent) {
        if (pocketDimensionManager.pocketDimension != event.location.world) return
        event.isCancelled = true
    }

    @EventHandler
    fun onPlayerDamage(event: EntityDamageEvent) {
        if (event.entity !is Player) return
        if (pocketDimensionManager.pocketDimension != event.entity.world) return
        event.isCancelled = true
    }

    @EventHandler
    fun onPlayerFall(event: PlayerMoveEvent) {
        if (pocketDimensionManager.pocketDimension != event.from.world
            || pocketDimensionManager.pocketDimension != event.to.world
        ) return
        if (event.from.chunk != event.to.chunk) {
            event.player.sendMessage("Something pulls you back from the darkness.")
            event.isCancelled = true
        }
        if (event.to.blockY < 0 || event.to.blockY > 256) {
            event.player.sendMessage("Something pulls you back from the darkness.")
            val safeLocation = event.player.location
            safeLocation.y = 128.0
            event.player.teleport(safeLocation)
        }
    }
}